//+------------------------------------------------------------------+
//|                                              Multi pair MACD.mq4 |
//|                                                           mladen |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      ""

#property indicator_separate_window
#property  indicator_buffers 3
#property  indicator_color1  Green
#property  indicator_color2  Green
#property  indicator_color3  Gray
#property  indicator_width1  2
#property  indicator_minimum 0

//---- input parameters
//
//
//
//

extern int       AtrPeriod   = 12;
extern int       barsPerPair = 100;
extern string    pairs       = "EURUSD;GBPUSD;USDCHF";
extern bool      equalize    = false;
extern string    text        = "color";
extern color     textColor   = Silver;
extern color     backColor   = C'48,48,48';

//---- buffers
//
//
//
//

double   ind_buffer1[];
double   ind_buffer2[];
double   ind_buffer5[];
double   ind_buffer7[];
double   ind_buffer8[];

//
//
//
//
//

double   minAtr;
double   maxAtr;

//
//
//
//
//

double   maxValues[];
string   aPairs[];
string   shortName;
string   labelNames = "multiMACD";
int      window;  
int      add;



//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(4);
   IndicatorDigits(Digits+1);
   SetIndexBuffer(0,ind_buffer1);
   SetIndexBuffer(1,ind_buffer2);
   SetIndexBuffer(2,ind_buffer5);
   SetIndexBuffer(3,ind_buffer7);

   //
   //
   //
   //
   //
   
      ArrayResize(ind_buffer8,barsPerPair);
      ArraySetAsSeries(ind_buffer8,true);

   //
   //
   //
   //
   //
      
   SetIndexStyle(0,DRAW_HISTOGRAM); 
   SetIndexStyle(1,DRAW_HISTOGRAM);

   //
   //
   //    convert pairs to array
   //
   //

      pairs = StringTrimLeft(StringTrimRight(pairs));
      if (StringSubstr(pairs,StringLen(pairs),1) != ";")
                       pairs = StringConcatenate(pairs,";");

         //
         //
         //
         //
         //                                   
            
         int s =  0;
         int i =  StringFind(pairs,";",s);
         string current;
         string temp;
            while (i > 0)
            {
               current = StringSubstr(pairs,s,i-s);
               if (iClose(current,0,0) > 0)
                  {
                     ArrayResize(aPairs,ArraySize(aPairs)+1);
                                 aPairs[ArraySize(aPairs)-1] = current;
                                 if (current == Symbol())
                                 {
                                       temp      = aPairs[0];
                                       aPairs[0] = current;
                                       aPairs[ArraySize(aPairs)-1] = temp;
                                 }                                       
                  }
                  s = i + 1;
                  i = StringFind(pairs,";",s);
            }

   //
   //
   //
   //
   //
 
      shortName = "Multi ATR ("+AtrPeriod+")";
      IndicatorShortName(shortName);
 
   //
   //
   //
   //
   //
   
   return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+

int deinit()
{
   for (int i = 0; i < ArraySize(aPairs); i++) { 
         ObjectDelete(labelNames+window+i);
         ObjectDelete(labelNames+window+i+i);
      }         
   return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   static bool init = false;
   int    limit     = ArraySize(aPairs);
   int    i,k;
   double max;
   
   //
   //
   //
   //
   //

   if (!init) {
        init      = true;
        window    = WindowFind(shortName);  
        shortName = "Multi ATR - "+window+" ("+AtrPeriod+")";
            IndicatorShortName(shortName);
            
            //
            //
            //
            //
            //
            
            ArrayResize(maxValues,limit);
      }            
      ArrayInitialize(maxValues,0);

   //
   //
   //
   //
   //

         minAtr =  100000;
         maxAtr = -100000;
                for(i=0,k=0; i<limit; i++) calculateAtr(aPairs[i],barsPerPair,k,i);
                for(i=0;     i<limit; i++) if (max < maxValues[i]) max = maxValues[i];
                for(i=0,k=0; i<limit; i++) plotGraph(aPairs[i],barsPerPair,k,i,max);

   //
   //
   //
   //
   //

   for (i=0;i<indicator_buffers;i++) SetIndexDrawBegin(i,Bars-k);
   return(0);
}


//+------------------------------------------------------------------+
//+                                                                  +
//+------------------------------------------------------------------+

void plotGraph(string symbol,int limit, int& shift,int element, double max)
{
   double   koef   = 1;
   bool     signalCrossing = false;
   
   //
   //
   //
   //
   //
   
   if (equalize) koef = max/maxValues[element];

   //
   //
   //
   //
   //

   for(int i=0; i<limit; i++) {   
      ind_buffer5[i+shift] = ind_buffer5[i+shift]*koef;
      
      //
      //
      //
      //
      
            if (ind_buffer7[i+shift] > ind_buffer7[i+shift+1]) {
                     ind_buffer1[i+shift] = ind_buffer7[i+shift]*koef;
                     ind_buffer2[i+shift] = EMPTY_VALUE;  }
            else {   ind_buffer2[i+shift] = ind_buffer7[i+shift]*koef;
                     ind_buffer1[i+shift] = EMPTY_VALUE; }
   }

   //
   //
   //
   //
   //


   createLabel(symbol,element,shift+limit+4,max,signalCrossing);
   
   //
   //
   //
   //
   //
   
   shift += (limit+5);
}

//+------------------------------------------------------------------+
//+                                                                  +
//+------------------------------------------------------------------+

void calculateAtr(string symbol,int limit,int& shift,int element)
{
   double max;
   int    i;

   //
   //
   //
   //
   //

   for(i=0; i<limit+add; i++) ind_buffer8[i]=iATR(symbol,0,AtrPeriod,i);
   for(i=0; i<limit; i++) {
      ind_buffer5[i+shift]=ind_buffer8[i];
      ind_buffer7[i+shift]=ind_buffer8[i];
      
      //
      //
      //
      //
      //

      max = ind_buffer5[i+shift];
            if (maxValues[element] < max)
                maxValues[element] = max;

      //
      //
      //
      //
      //
                              
         if (maxAtr < max) maxAtr = max;
         if (minAtr > max) minAtr = max;
   }            

   //
   //
   //
   //
   //

   for (i=0;i<6;i++) {
         ind_buffer1[shift+limit+i] = EMPTY_VALUE;                     
         ind_buffer2[shift+limit+i] = EMPTY_VALUE;                     
         ind_buffer5[shift+limit+i] = EMPTY_VALUE;                     
      }         
   shift += (limit+5);
   return;
}



//+------------------------------------------------------------------+
//+                                                                  +
//+------------------------------------------------------------------+
//
//
//
//
//

void createLabel(string symbol,int element, int shift, double max, bool signalCrossing)
{
   string name = labelNames+window+element;
   double price1;
   double price2;
   
   
   //
   //
   //
   //
   //
   
   price1 = maxAtr;
   price2 = 0;
   
   //
   //
   //
   //
   //
   
   if (ObjectFind(name) == -1)
      {
         ObjectCreate(name,OBJ_TEXT,window,0,0);
         ObjectSet(name,OBJPROP_ANGLE,90);
         ObjectSetText(name,symbol);
      }
      ObjectSet(name,OBJPROP_TIME1 ,Time[shift]);
      ObjectSet(name,OBJPROP_PRICE1,(price1+price2)/2);
      if (signalCrossing)
           ObjectSet(name,OBJPROP_COLOR,Gold);
      else ObjectSet(name,OBJPROP_COLOR,textColor);

   //
   //
   //
   //
   //

 
   name = labelNames+window+element+element;
   if (ObjectFind(name) == -1)
      {
         ObjectCreate(name,OBJ_RECTANGLE,window,0,0,0,0);
         ObjectSet(name,OBJPROP_COLOR,backColor);
      }         
      ObjectSet(name,OBJPROP_TIME1,Time[shift]);
      ObjectSet(name,OBJPROP_PRICE1,price1);
      ObjectSet(name,OBJPROP_TIME2,Time[shift-4]);
      ObjectSet(name,OBJPROP_PRICE2,price2);
}

